/** 
 * <p>Titre : Query</p>
 * <p>Description : Classe utilitaire pour les accs base de donnes</p>
 * <p>Copyright : Copyright (c) 2004</p>
 * <p>Socit : Universit Nancy 2</p>
 * @author Olivier Ziller & Cdric Champmartin
 * @version 1.0
 */
  
package org.esupportail.portal.utils.database;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.apache.log4j.Priority;
import org.jasig.portal.RDBMServices;
import org.jasig.portal.services.LogService;

/**
 * Classe d'abstraction pour accs aux base de donnes.
 * L'objectif est de simplifier les accs les plus courants  la librairie JDBC et 
 * de centraliser les exceptions qu'elles peuvent soulever.
 */
public class Query {
	
	// Connexion  utiliser
	private Connection con = null;
	// Objet qui contient la connexion et le nombre de processus l'utilisant
	//private Connect connect = null;
	// Statement
	private RDBMServices.PreparedStatement statement = null;
	// Resulset
	private ResultSet resultset = null;
	// Ordre SQL
	private String sql = null;
	// chaine de caractre dsignant le prfix dans les requtes SQL
	private static final String PREFIX = "__";
	// Mode debuggage ?
	private boolean debug = false;
	// Type de log pour le debuggage
	protected Priority nivlog = LogService.DEBUG;
	// Y-a-t-il un commit automatique aprs les insert/delete/update
 	private boolean autoCommit = true;
	// type de l'objet Connection en cours
	private String type;
	// Niveau d'isolation des transactions
	private int isolation = Connection.TRANSACTION_READ_COMMITTED;
	
	/**
	 * Constructeur 
	 * @deprecated
	 * @param pool pool de connexion  utiliser pour la requte
	 */
	public Query(String pool) {
		setCon(RDBMServices.getConnection(pool));
	}
	
	/**
	 * Constructeur 
	 * @param connexion objet connexion  utiliser pour la requte
	 */
	public Query(Database db) throws SQLException {
		if (db != null)
			setCon(db);
		else {
			LogService.log(LogService.ERROR ,"Query::Query(Database) : Objet Database vide");
			throw new SQLException();
		}
				
	}
	
	public Query(Database db, Priority logLevel) throws SQLException {
		this(db);
		nivlog = logLevel;
	}
		
	/**
	 * Fermeture de la requete : resultset, statement et connexion
	 *
	 */
	public Exception close() {
		Exception res = null;
		
		try {
			closeRs();
		}
		catch (Exception e) {
				res = e;
				logException(e, "close resultset");
		}
		try {
			closeStmt();
		}
		catch (Exception e) {
			res = e;
			logException(e, "close statement");
		}
		try {
			if (getCon() != null) {
				if (type.equals("JNDI")) 
					RDBMServices.releaseConnection(getCon());
				else 
					getCon().close();
			}
		}
		catch (Exception e) {
			res = e;
			logException(e, "close connection");
		}
		return res;
	}
	
	/**
	 * Fermeture du resultset associ  l'objet
	 * @throws SQLException
	 */
	private void closeRs() throws SQLException {
		if (getRs() != null)
			getRs().close();
	}
	
	/**
	 * Fermeture du statement associ  l'objet
	 * @throws SQLException
	 */
	private void closeStmt() throws SQLException {
		if (getStmt() != null)
			getStmt().close();
	}
	/**
	 * rcupre la connexion associe  l'objet
	 */
	private Connection getCon() {
		return con;
	}

	/**
	 * rcupre le resultset associ  l'objet
	 */
	public ResultSet getRs() {
		return resultset;
	}

	/**
	 * retourne le statement associ  l'objet
	 */
	public RDBMServices.PreparedStatement getStmt() {
		return statement;
	}

	/**
	 * Log les exceptions
	 * 
	 * @param exception
	 * @param comment commentaire  afficher dans le log
	 */
	private void logException(Exception exception, String comment) {
		LogService.log(LogService.ERROR ,"Query:logExeception() : " + comment +
		  "(" + exception.getMessage() + ")");
	}

	/**
	 * rcupre le sql de l'objet
	 */
	private String getSql() {
		return sql;
	}

	/**
	 * Positionnement de la requte SQL de l'objet
	 * @param sql
	 * @throws SQLException
	 */
	public void setSql(String sql) throws SQLException {
		this.sql = sql;
		// Fermeture ventuelle de la requte en cours
		closeRs();
		closeStmt();
		// Puis ouverture
		setStmt(new RDBMServices.PreparedStatement(getCon(), sql));
		statement.clearParameters();
	}
	
	/**
	 * Positionne le sql en positionnant un prfixe devant les tables de la requte
	 * @param sql requte SQL
	 * @param prefix prfixe  ajouter en lieu et place de __
	 * @throws SQLException
	 */
	public void setSql(String sql, String prefix) throws SQLException {
		setSql(sql.replaceAll(PREFIX,prefix));
	}

	/**
	 * Ouverture de la requte
	 * @throws SQLException
	 */
	public void select() throws SQLException {
		if (getStmt() == null)
			throw new SQLException("select : le statement est null");
		setRs(getStmt().executeQuery());
	}

	/**
	 * Commit
	 */
	public void commit() {
		try {
			con.commit();	
		} catch (SQLException e) {
			// TODO Bloc catch auto-gnr
			logException(e, "commit");
		}
	}
	
	/**
	 * rollback
	 */
	public void rollback() {
		try {
			con.rollback();	
		} catch (SQLException e) {
			// TODO Bloc catch auto-gnr
			logException(e, "rollback");
		}
	}
	
	/**
	 * Execution d'un ordre SQLs
	 * @param mode : insert/delete/update (ne sert que pour les logs)
	 * @return
	 * @throws SQLException
	 */
	private int execSql(String mode) throws SQLException {
		if(!autoCommit) {
			con.setAutoCommit(false);
			con.setTransactionIsolation(isolation);
		}
		if (getStmt() == null)
			throw new SQLException(mode + " : le statement est null");
		int res = (getStmt().executeUpdate());
		return res;
	}
	
	/**
	 * Insertion
	 * @throws SQLException
	 */
	public int insert() throws SQLException {
		return execSql("insert");
	}
	
	/**
	 * Delete
	 * @throws SQLException
	 */
	public int delete() throws SQLException {
		return execSql("delete");
	}
	
	/**
	 * Update
	 * @throws SQLException
	 */
	public int update() throws SQLException {
		return execSql("update");
	}
	
	/**
	 * Positionnement de la requte et ouverture de celle-ci en une opration
	 * @param sql requte sql
	 * @throws SQLException
	 */
	public void select(String sql) throws SQLException {
		setSql(sql);
		select();
	}
	
	/**
	 * Postionnement de la connexion de l'objet
	 * @param connection
	 */
	private void setCon(Connection connection) {
		con = connection;
		type = "JNDI";
	}
	
	/**
	 * Postionnement de la connexion de l'objet
	 * @param db objet Database
	 */
	private void setCon(Database db) {
		// cas pool Tomcat			
		if (db.getType().toUpperCase().equals("JNDI")) {
			con = RDBMServices.getConnection(db.getUrl());
			if (con == null) {
				LogService.log(LogService.ERROR ,"Query::setCon(Database) : connexion vide");
			}	
			type = "JNDI";
		} // cas JDBC simple
		else if (db.getType().toUpperCase().equals("JDBC")) {
			type = "JDBC";
			createConnection(db);
			// Si l'objet connexion n'est pas null mais qu'il n'est plus valide, on le recre		
			try{
				if(con.isClosed()){
					createConnection(db);
				}
			}
			catch(SQLException e){
				LogService.log(LogService.ERROR, "Query::setCon() Erreur de connexion  l'url : " + db.getUrl() + ";" + e);
			}				
		}
		else { // pas de type prcis
			LogService.log(LogService.ERROR, "Query::setCon() Erreur de connexion  l'url : " + db.getUrl() + "; Pas de type de connexion prcis");
		}
	}
	
	/**
	 * Cration d'une connexion JDBC 
	 * @param db objet database
	 */
	private void createConnection(Database db){
		try{
			Class.forName(db.getDriverClassName()).newInstance();
			con = DriverManager.getConnection(db.getUrl(),db.getUsername(),db.getPassword());
			String key = db.getDriverClassName()+":"+db.getUrl()+":"+db.getUsername()+":"+db.getPassword();
			if (con == null){
				LogService.log(LogService.ERROR, "Query::createConnection() Erreur de connexion  l'url : " + db.getUrl());
			}
		}
		catch(Exception e){
			LogService.log(LogService.ERROR, "Query::createConnection() Erreur de connexion  l'url : " + db.getUrl() + ";" + e);
		}
	}

	/**
	 * @param set
	 */
	private void setRs(ResultSet resultset) {
		this.resultset = resultset;
	}

	/**
	 * @param statement
	 */
	private void setStmt(RDBMServices.PreparedStatement statement) {
		this.statement = statement;
	}

	/**
	 * @return
	 */
	private boolean isDebug() {
		return debug;
	}

	/**
	 * @param b
	 */
	public void setDebug(boolean b) {
		debug = b;
	}

	/**
	 * Cette fonction permet de logger le fonctionnement de la classe (si isDebug)
	 * Le niveau de log est dtermin par la variable nivDebug
	 * @param msg
	 */
	private void debug(String msg) {
		if (isDebug())
			log(msg);			
	}
	
	/**
	 * l'autocommit est-il actif?
	 * @return
	 */
	public boolean isAutoCommit() {
		return autoCommit;
	}

	/**
	 * Positionnement de l'autocommit
	 * @param b
	 */
	public void setAutoCommit(boolean autocommit) {
		this.autoCommit = autocommit; 
	}

	/**
	 * Retourne une chaine entre '
	 * @param s chaine  mettre entre '
	 * @return
	 */
	public static String quotedStr(String s) {
		return "'" + s + "'";
	}
	
	public void setTransactionIsolation(int level) {
		isolation = level;
	}

	 /**
	  * Envoi d'un message dans le fichier de log pour le niveau de log par dfaut
	  * @param logMessage
	  */
	 public void log(String logMessage) {
	 	log(nivlog, logMessage);
	 }
	 
	 /**
	  * Envoi d'un message dans le fichier de log selon le niveau pass en paramtre
	  * @param level
	  * @param logMessage
	  */
	 public void log(Priority level, String logMessage) {
	 	LogService.log(level, logMessage);
	 }	
}
